import java.util.Random;

public class AirplaneMixer<T extends EntityAirPlane, U extends IDrawingPorthole> {
    private T[] airplanes;
    private U[] portholes;

    public AirplaneMixer(int countAirplane, int countPortholes) {
        airplanes = (T[])(new EntityAirPlane[countAirplane]);
        portholes = (U[])(new IDrawingPorthole[countPortholes]);
    }

    public int add(T a) {
        int index = 0;
        while(index < airplanes.length && airplanes[index] != null) index++;

        if(index == airplanes.length) return -1;

        airplanes[index] = a;
        return index;
    }

    public int add(U e) {
        int index = 0;
        while(index < portholes.length && portholes[index] != null) index++;

        if(index == portholes.length) return -1;

        portholes[index] = e;
        return index;
    }

    public DrawingEntityPlain constructAirplane(int width, int height) {
        Random rnd = new Random();

        DrawingEntityPlain air;

        T selectedAirplane = airplanes[rnd.nextInt(0, airplanes.length)];
        U selectedPortholes = portholes[rnd.nextInt(0, portholes.length)];

        DrawingEntityPlain result;

        if(selectedAirplane instanceof EntityRadarPlain) {
            result = new DrawingRadarPlain(selectedAirplane, selectedPortholes);
        }
        else result = new DrawingEntityPlain(selectedAirplane, selectedPortholes);

        result.SetPosition(10, 10, width, height);
        return result;
    }
}
