import java.awt.*;
import java.util.Random;
class DrawingEntityPlain
{
    public EntityAirPlane Airplane;
    public IDrawingPorthole drawingPortholes;
    protected  float _startPosX;
    protected  float _startPosY;

    private int _pictureWidth = -1;
    private int _pictureHeight = -1;

    private int _AirplaneWidth = 240;
    private int _AirplaneHeight = 150;

    private void peekRandomPortholes(Color color) {
        Random rnd = new Random();
        int randPorthole = rnd.nextInt(1, 4);

        switch(randPorthole) {
            case 1:
                drawingPortholes = new DrawingPorthole(rnd.nextInt(1, 35), color);
                break;
            case 2:
                drawingPortholes = new DrawingSquarePorthole(rnd.nextInt(1, 35), color);
                break;
            case 3:
                drawingPortholes = new DrawingTrianglePorthole(rnd.nextInt(1, 35), color);
                break;
        }
    }

    public DrawingEntityPlain(int speed, float weight, Color bodyColor)
    {
        Airplane = new EntityAirPlane(speed, weight, bodyColor);
        peekRandomPortholes(bodyColor);
    }

    public DrawingEntityPlain(int speed, float weight, Color bodyColor, int airPlaneWidth, int airPlaneHeight)
    {
        this(speed, weight, bodyColor);
        _AirplaneWidth = airPlaneWidth;
        _AirplaneHeight = airPlaneHeight;
    }
    public DrawingEntityPlain(EntityAirPlane entityAirPlane, IDrawingPorthole iDrawingPorthole){
        Airplane = entityAirPlane;
        drawingPortholes = iDrawingPorthole;
    }

    public void SetPosition(int x, int y, int width, int height)
    {
        if (width < _AirplaneWidth || height < _AirplaneHeight) return;

        if (x + _AirplaneWidth > width || x < 0) return;
        if (y + _AirplaneHeight > height || y < 0) return;

        _startPosX = x;
        _startPosY = y;

        _pictureWidth = width;
        _pictureHeight = height;
    }

    public void MoveTransport(Direction direction)
    {
        if (_pictureWidth == -1 || _pictureHeight == -1)
        {
            return;
        }


        switch (direction)
        {
            case Right:
                if (_startPosX + _AirplaneWidth + Airplane.Step < _pictureWidth)
                {
                    _startPosX += Airplane.Step;
                }
                break;
            case Left:
                if (_startPosX - Airplane.Step > 0)
                {
                    _startPosX -= Airplane.Step;
                }
                break;
            case Up:
                if (_startPosY - Airplane.Step > 0)
                {
                    _startPosY -= Airplane.Step;
                }
                break;
            case Down:
                if (_startPosY + _AirplaneHeight + Airplane.Step < _pictureHeight)
                {
                    _startPosY += Airplane.Step;
                }
                break;
        }

    }

    public void DrawTransport(Graphics2D g)
    {
        if (_pictureWidth == -1 || _pictureHeight == -1)
        {
            return;
        }

        g.setPaint(Color.black);
        g.drawRect( (int)_startPosX, (int)_startPosY, 40, 60);
        g.drawRect( (int)_startPosX, (int)_startPosY + 60, 200, 60);
        g.drawRect( (int)_startPosX+100, (int)_startPosY + 80, 40, 30);
        //koleso1
        g.drawRect( (int)_startPosX + 60, (int)_startPosY + 120, 10, 20);
        g.drawOval( (int)_startPosX+56,(int) _startPosY+140, 18, 18);
        //koleso2
        g.drawRect( (int)_startPosX + 160, (int)_startPosY + 120, 10, 20);
        g.drawOval( (int)_startPosX + 156, (int)_startPosY + 140, 18, 18);

        //Korpys
        g.setPaint(Airplane.BodyColor);
        g.fillRect((int)_startPosX+6, (int)_startPosY + 66, 188, 48);
        g.fillRect( (int)_startPosX+2, (int)_startPosY+2, 38, 58);

        //krilya
        g.setPaint(Color.black);
        g.fillRect((int)_startPosX + 60, (int)_startPosY + 80, 80, 16);

        //cabina
        g.setPaint(Color.blue);
        g.fillRect( (int)_startPosX + 202, (int)_startPosY + 82, 38, 28);

        drawingPortholes.draw(g, (int)_startPosX+6, (int)_startPosY+70);
    }

    public void ChangeBorders(int width, int height)
    {
        _pictureWidth = width;
        _pictureHeight = height;
        if (_pictureWidth <= _AirplaneWidth || _pictureHeight <= _AirplaneHeight)
        {
            _pictureWidth = -1;
            _pictureHeight = -1;
            return;
        }
        if (_startPosX + _AirplaneWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _AirplaneWidth;
        }
        if (_startPosY + _AirplaneHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _AirplaneHeight;
        }
    }
    public Point getLeftTop() {
        return new Point((int)_startPosX, (int)_startPosY);
    }

    public Point getRightBottom() {
        return new Point((int)_startPosX + _AirplaneWidth, (int)_startPosY + _AirplaneHeight);
    }

}